/*
    SDL - Simple DirectMedia Layer
    Copyright (C) 1997-2009 Sam Lantinga

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Sam Lantinga
    slouken@libsdl.org
*/
module org.libsdl.events;

import org.libsdl.stdinc;
import org.libsdl.keyboard;

immutable int SDL_RELEASED = 0;
immutable int SDL_PRESSED  = 1;

enum SDL_EventType
{
	SDL_NOEVENT = 0,
	SDL_ACTIVEEVENT,
	SDL_KEYDOWN,
	SDL_KEYUP,
	SDL_MOUSEMOTION,
	SDL_MOUSEBUTTONDOWN,
	SDL_MOUSEBUTTONUP,
	SDL_JOYAXISMOTION,
	SDL_JOYBALLMOTION,
	SDL_JOYHATMOTION,
	SDL_JOYBUTTONDOWN,
	SDL_JOYBUTTONUP,
	SDL_QUIT,
	SDL_SYSWMEVENT,
	SDL_EVENT_RESERVEDA,
	SDL_EVENT_RESERVEDB,
	SDL_VIDEORESIZE,
	SDL_VIDEOEXPOSE,
	SDL_EVENT_RESERVED2,
	SDL_EVENT_RESERVED3,
	SDL_EVENT_RESERVED4,
	SDL_EVENT_RESERVED5,
	SDL_EVENT_RESERVED6,
	SDL_EVENT_RESERVED7,
	SDL_USEREVENT = 24,
	SDL_NUMEVENTS = 32,
}

alias SDL_EventType.SDL_NOEVENT SDL_NOEVENT;
alias SDL_EventType.SDL_ACTIVEEVENT SDL_ACTIVEEVENT;
alias SDL_EventType.SDL_KEYDOWN SDL_KEYDOWN;
alias SDL_EventType.SDL_KEYUP SDL_KEYUP;
alias SDL_EventType.SDL_MOUSEMOTION SDL_MOUSEMOTION;
alias SDL_EventType.SDL_MOUSEBUTTONDOWN SDL_MOUSEBUTTONDOWN;
alias SDL_EventType.SDL_MOUSEBUTTONUP SDL_MOUSEBUTTONUP;
alias SDL_EventType.SDL_JOYAXISMOTION SDL_JOYAXISMOTION;
alias SDL_EventType.SDL_JOYBALLMOTION SDL_JOYBALLMOTION;
alias SDL_EventType.SDL_JOYHATMOTION SDL_JOYHATMOTION;
alias SDL_EventType.SDL_JOYBUTTONDOWN SDL_JOYBUTTONDOWN;
alias SDL_EventType.SDL_JOYBUTTONUP SDL_JOYBUTTONUP;
alias SDL_EventType.SDL_QUIT SDL_QUIT;
alias SDL_EventType.SDL_SYSWMEVENT SDL_SYSWMEVENT;
alias SDL_EventType.SDL_EVENT_RESERVEDA SDL_EVENT_RESERVEDA;
alias SDL_EventType.SDL_EVENT_RESERVEDB SDL_EVENT_RESERVEDB;
alias SDL_EventType.SDL_VIDEORESIZE SDL_VIDEORESIZE;
alias SDL_EventType.SDL_VIDEOEXPOSE SDL_VIDEOEXPOSE;
alias SDL_EventType.SDL_EVENT_RESERVED2 SDL_EVENT_RESERVED2;
alias SDL_EventType.SDL_EVENT_RESERVED3 SDL_EVENT_RESERVED3;
alias SDL_EventType.SDL_EVENT_RESERVED4 SDL_EVENT_RESERVED4;
alias SDL_EventType.SDL_EVENT_RESERVED5 SDL_EVENT_RESERVED5;
alias SDL_EventType.SDL_EVENT_RESERVED6 SDL_EVENT_RESERVED6;
alias SDL_EventType.SDL_EVENT_RESERVED7 SDL_EVENT_RESERVED7;
alias SDL_EventType.SDL_USEREVENT SDL_USEREVENT;
alias SDL_EventType.SDL_NUMEVENTS SDL_NUMEVENTS;

template SDL_EVENTMASK(uint event)
{
	immutable uint SDL_EVENTMASK = 1 << event;
}

enum SDL_EventMask : uint
{
	SDL_ACTIVEEVENTMASK     = SDL_EVENTMASK!(SDL_EventType.SDL_ACTIVEEVENT),
	SDL_KEYDOWNMASK         = SDL_EVENTMASK!(SDL_EventType.SDL_KEYDOWN),
	SDL_KEYUPMASK           = SDL_EVENTMASK!(SDL_EventType.SDL_KEYUP),
	SDL_KEYEVENTMASK        = SDL_EVENTMASK!(SDL_EventType.SDL_KEYDOWN) | SDL_EVENTMASK!(SDL_EventType.SDL_KEYUP),
	SDL_MOUSEMOTIONMASK     = SDL_EVENTMASK!(SDL_EventType.SDL_MOUSEMOTION),
	SDL_MOUSEBUTTONDOWNMASK = SDL_EVENTMASK!(SDL_EventType.SDL_MOUSEBUTTONDOWN),
	SDL_MOUSEBUTTONUPMASK   = SDL_EVENTMASK!(SDL_EventType.SDL_MOUSEBUTTONUP),
	SDL_MOUSEEVENTMASK      = SDL_EVENTMASK!(SDL_EventType.SDL_MOUSEMOTION) | SDL_EVENTMASK!(SDL_EventType.SDL_MOUSEBUTTONDOWN) | SDL_EVENTMASK!(SDL_EventType.SDL_MOUSEBUTTONUP),
	SDL_JOYAXISMOTIONMASK   = SDL_EVENTMASK!(SDL_EventType.SDL_JOYAXISMOTION),
	SDL_JOYBALLMOTIONMASK   = SDL_EVENTMASK!(SDL_EventType.SDL_JOYBALLMOTION),
	SDL_JOYHATMOTIONMASK    = SDL_EVENTMASK!(SDL_EventType.SDL_JOYHATMOTION),
	SDL_JOYBUTTONDOWNMASK   = SDL_EVENTMASK!(SDL_EventType.SDL_JOYBUTTONDOWN),
	SDL_JOYBUTTONUPMASK     = SDL_EVENTMASK!(SDL_EventType.SDL_JOYBUTTONUP),
	SDL_JOYEVENTMASK        = SDL_EVENTMASK!(SDL_EventType.SDL_JOYAXISMOTION) | SDL_EVENTMASK!(SDL_EventType.SDL_JOYBALLMOTION) | SDL_EVENTMASK!(SDL_EventType.SDL_JOYHATMOTION) | SDL_EVENTMASK!(SDL_EventType.SDL_JOYBUTTONDOWN) | SDL_EVENTMASK!(SDL_EventType.SDL_JOYBUTTONUP),
	SDL_VIDEORESIZEMASK     = SDL_EVENTMASK!(SDL_EventType.SDL_VIDEORESIZE),
	SDL_VIDEOEXPOSEMASK     = SDL_EVENTMASK!(SDL_EventType.SDL_VIDEOEXPOSE),
	SDL_QUITMASK            = SDL_EVENTMASK!(SDL_EventType.SDL_QUIT),
	SDL_SYSWMEVENTMASK      = SDL_EVENTMASK!(SDL_EventType.SDL_SYSWMEVENT),
}

alias SDL_EventMask.SDL_ACTIVEEVENTMASK SDL_ACTIVEEVENTMASK;
alias SDL_EventMask.SDL_KEYDOWNMASK SDL_KEYDOWNMASK;
alias SDL_EventMask.SDL_KEYUPMASK SDL_KEYUPMASK;
alias SDL_EventMask.SDL_KEYEVENTMASK SDL_KEYEVENTMASK;
alias SDL_EventMask.SDL_MOUSEMOTIONMASK SDL_MOUSEMOTIONMASK;
alias SDL_EventMask.SDL_MOUSEBUTTONDOWNMASK SDL_MOUSEBUTTONDOWNMASK;
alias SDL_EventMask.SDL_MOUSEBUTTONUPMASK SDL_MOUSEBUTTONUPMASK;
alias SDL_EventMask.SDL_MOUSEEVENTMASK SDL_MOUSEEVENTMASK;
alias SDL_EventMask.SDL_JOYAXISMOTIONMASK SDL_JOYAXISMOTIONMASK;
alias SDL_EventMask.SDL_JOYBALLMOTIONMASK SDL_JOYBALLMOTIONMASK;
alias SDL_EventMask.SDL_JOYHATMOTIONMASK SDL_JOYHATMOTIONMASK;
alias SDL_EventMask.SDL_JOYBUTTONDOWNMASK SDL_JOYBUTTONDOWNMASK;
alias SDL_EventMask.SDL_JOYBUTTONUPMASK SDL_JOYBUTTONUPMASK;
alias SDL_EventMask.SDL_JOYEVENTMASK SDL_JOYEVENTMASK;
alias SDL_EventMask.SDL_VIDEORESIZEMASK SDL_VIDEORESIZEMASK;
alias SDL_EventMask.SDL_VIDEOEXPOSEMASK SDL_VIDEOEXPOSEMASK;
alias SDL_EventMask.SDL_QUITMASK SDL_QUITMASK;
alias SDL_EventMask.SDL_SYSWMEVENTMASK SDL_SYSWMEVENTMASK;
immutable uint SDL_ALLEVENTS = 0xFFFFFFFF;

struct SDL_ActiveEvent
{
	Uint8 type;
	Uint8 gain;
	Uint8 state;
}

struct SDL_KeyboardEvent
{
	Uint8 type;
	Uint8 which;
	Uint8 state;
	SDL_keysym keysym;
}

struct SDL_MouseMotionEvent
{
	Uint8 type;
	Uint8 which;
	Uint8 state;
	Uint16 x, y;
	Sint16 xrel;
	Sint16 yrel;
}

struct SDL_MouseButtonEvent
{
	Uint8 type;
	Uint8 which;
	Uint8 button;
	Uint8 state;
	Uint16 x, y;
}

struct SDL_JoyAxisEvent
{
	Uint8 type;
	Uint8 which;
	Uint8 axis;
	Sint16 value;
}

struct SDL_JoyBallEvent
{
	Uint8 type;
	Uint8 which;
	Uint8 ball;
	Sint16 xrel;
	Sint16 yrel;
}

struct SDL_JoyHatEvent
{
	Uint8 type;
	Uint8 which;
	Uint8 hat;
	Uint8 value;
}

struct SDL_JoyButtonEvent
{
	Uint8 type;
	Uint8 which;
	Uint8 button;
	Uint8 state;
}

struct SDL_ResizeEvent
{
	Uint8 type;
	int w;
	int h;
}

struct SDL_ExposeEvent
{
	Uint8 type;
}

struct SDL_QuitEvent
{
	Uint8 type;
}

struct SDL_UserEvent
{
	Uint8 type;
	int code;
	void* data1;
	void* data2;
}

struct SDL_SysWMmsg;

struct SDL_SysWMEvent
{
	Uint8 type;
	SDL_SysWMmsg* msg;
}

union SDL_Event
{
	Uint8 type;
	SDL_ActiveEvent active;
	SDL_KeyboardEvent key;
	SDL_MouseMotionEvent motion;
	SDL_MouseButtonEvent button;
	SDL_JoyAxisEvent jaxis;
	SDL_JoyBallEvent jball;
	SDL_JoyHatEvent jhat;
	SDL_JoyButtonEvent jbutton;
	SDL_ResizeEvent resize;
	SDL_ExposeEvent expose;
	SDL_QuitEvent quit;
	SDL_UserEvent user;
	SDL_SysWMEvent syswm;
}

extern (C)
{
	void SDL_PumpEvents();
}

enum SDL_eventaction
{
	SDL_ADDEVENT,
	SDL_PEEKEVENT,
	SDL_GETEVENT
}

alias SDL_eventaction.SDL_ADDEVENT SDL_ADDEVENT;
alias SDL_eventaction.SDL_PEEKEVENT SDL_PEEKEVENT;
alias SDL_eventaction.SDL_GETEVENT SDL_GETEVENT;

extern (C)
{
	int SDL_PeepEvents(SDL_Event* events, int numevents, SDL_eventaction action, Uint32 mask);
	int SDL_PollEvent(SDL_Event* event);
	int SDL_WaitEvent(SDL_Event* event);
	int SDL_PushEvent(SDL_Event* event);
	alias int function(const(SDL_Event)* event) SDL_EventFilter;
	void SDL_SetEventFilter(SDL_EventFilter filter);
	SDL_EventFilter SDL_GetEventFilter();
}

immutable int SDL_QUERY   = -1;
immutable int SDL_IGNORE  = 0;
immutable int SDL_DISABLE = 0;
immutable int SDL_ENABLE  = 1;

extern (C)
{
	Uint8 SDL_EventState(Uint8 type, int state);
}
